<?php
if (!isset($siledap_folder)) {
	require_once("../config.php");
}
require_once($siledap_folder."/LdapEntry.php");

class LdapManager {
    var $ldap;
    var $ldapResource;
    
    function LdapManager($server,$port) {
        $this->ldap=ldap_connect($server,$port);
    }
    
    function getLdapLink() {
    	return $this->ldap;
    }
    
    function setOption($option,$value) {
        ldap_set_option($this->ldap,$option,$value);
    }
    
    function connectLdap($user,$pass) {
        if (strcmp($user,"")==0) {
            $this->ldapResource=ldap_bind($this->ldap);
        }
        else {
            $this->ldapResource=ldap_bind($this->ldap,$user,$pass);
        }
    }

    function replaceEntry($dn,$listOldAttributes,$objClassList,$attrList) {
		$namesAttrNew=array_keys($attrList);
		foreach ($listOldAttributes as $nameOldAttr) {
			if (!in_array($nameOldAttr,$namesAttrNew)) {
				$attrList[$nameOldAttr]=array();
			}
		}
		$attrList["objectClass"]=array();
		foreach ($objClassList as $objName) {
			$attrList["objectClass"][]=$objName;
		}
		error_log(print_r($listOldAttributes,true));
		error_log(print_r($attrList,true));
		return ldap_modify($this->ldap,$dn,$attrList);
	}

    function addEntry($dn,$objClassList,$attrList) {
        $info=array();
        foreach ($objClassList as $objClass) {
            $info['objectClass'][]=$objClass;
        }
        foreach ($attrList as $nameAttr => $valuesAttr) {
            foreach ($valuesAttr as $valueAttr) {
                $info[$nameAttr][]=$valueAttr;
            }
        }
        $res=ldap_add($this->ldap,$dn,$info);
        return $res;
    }
    
    function delEntry($dn,$delChildren) {
        if($delChildren == false){
            return ldap_delete($this->ldap,$dn);
        } 
        else {
            $sr=ldap_list($this->ldap,$dn,"ObjectClass=*",array(""));
            $info = ldap_get_entries($this->ldap, $sr);
            for($i=0;$i<$info['count'];$i++){
                $result=$this->delEntry($info[$i]['dn'],$delChildren);
                if(!$result){
                    return $result;
                }
            }
            return ldap_delete($this->ldap,$dn);
        }
    }
    
    function getErrorMessage() {
        return ldap_error($this->ldap);
    }
    
    function getEntry($dn,$attrs=array()) {
        $newEntry=new LdapEntry($dn);
        $res=ldap_read($this->ldap,$dn,"(objectClass=*)",$attrs);
        $entry=ldap_first_entry($this->ldap,$res);
        if ($entry!=false) {
            $listAttr=ldap_get_attributes($this->ldap,$entry);
            for ($i=0; $i<$listAttr["count"]; $i++) {
                if (strpos($listAttr[$i],";binary")===false) {
                    $listValues=ldap_get_values($this->ldap,$entry,$listAttr[$i]);
                }
                else {
                    $listValues=ldap_get_values_len($this->ldap,$entry,$listAttr[$i]);
                }
                for ($j=0; $j<$listValues["count"]; $j++) {
                    $newEntry->addAttribute($listAttr[$i],$listValues[$j]);
                }
            }
        }
        return $newEntry;
    }
    
    function getBinaryAttribute($dn,$nameAttr) {
        $attrs=array($nameAttr);
        $res=ldap_read($this->ldap,$dn,"(objectClass=*)",$attrs);
        $entry=ldap_first_entry($this->ldap,$res);
        return ldap_get_values_len($this->ldap,$entry,$nameAttr);
    }
    
    function search($dnbase,$filter,$oneLevel=false,$selAttr=array()) {
        $toReturn=array();
        if ($oneLevel) {
            $res=ldap_list($this->ldap,$dnbase,$filter,$selAttr);
        }
        else {
            $res=ldap_search($this->ldap,$dnbase,$filter,$selAttr);
        }
        for ($entry=ldap_first_entry($this->ldap, $res); $entry!=false; $entry=ldap_next_entry($this->ldap,$entry)) {
            $dnEntry=ldap_get_dn($this->ldap,$entry);
            $newEntry=new LdapEntry($dnEntry);
            if (count($selAttr)==0) {
                $listAttr=ldap_get_attributes($this->ldap,$entry);
                $listAttrLength=$listAttr["count"];
            }
            else {
                $listAttr=$selAttr;
                if (array_search('dn',$listAttr)!==false) {
                    unset($listAttr[array_search('dn',$listAttr)]);
                }
                $listAttrLength=count($listAttr);
            }
            for ($i=0; $i<$listAttrLength; $i++) {
                if (strpos($listAttr[$i],";binary")===false) {
                	$listValues=ldap_get_values($this->ldap,$entry,$listAttr[$i]);
                }
                else {
                    $listValues=ldap_get_values_len($this->ldap,$entry,$listAttr[$i]);
                }
                for ($j=0; $j<$listValues["count"]; $j++) {
                    $newEntry->addAttribute($listAttr[$i],$listValues[$j]);
                }
            }
            array_push($toReturn,$newEntry);
        }
        return $toReturn;
    }
}
?>
